@@ -1,7 +1,7 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Helper.c,v 1.147 2005/02/18 01:52:59 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Helper.c,v 1.158 2006/12/10 15:58:27 tsi Exp $ */
 
 /*
- * Copyright (c) 1997-2005 by The XFree86 Project, Inc.
+ * Copyright (c) 1997-2006 by The XFree86 Project, Inc.
  * All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -57,7 +57,7 @@
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  *  1. Redistributions of source code must retain the above copyright
  *     notice, this list of conditions, and the following disclaimer.
  *
@@ -65,7 +65,7 @@
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
- * 
+ *
  *  3. The end-user documentation included with the redistribution,
  *     if any, must include the following acknowledgment: "This product
  *     includes software developed by X-Oz Technologies
@@ -89,7 +89,7 @@
  * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
  * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- * 
+ *
  */
 
 /*
@@ -101,7 +101,7 @@
  * different drivers.
  */
 
-#include "X.h"
+#include <X11/X.h>
 #include "os.h"
 #include "servermd.h"
 #include "pixmapstr.h"
@@ -131,33 +131,11 @@
 static int xf86ScrnInfoPrivateCount = 0;
 
 #ifdef XFree86LOADER
-pointer *deferredUnloadList = NULL;
-int numDeferredUnloads;
-
-void
-xf86DoDeferredUnloads()
-{
-    int i;
-
-    for (i = 0; i < numDeferredUnloads; i++)
-	UnloadModule(deferredUnloadList[i]);
-    xfree(deferredUnloadList);
-    numDeferredUnloads = 0;
-}
-
-static void
-DeferredUnloadModule(pointer module)
-{
-    numDeferredUnloads++;
-    deferredUnloadList =
-	xnfrealloc(deferredUnloadList, numDeferredUnloads * sizeof(pointer));
-    deferredUnloadList[numDeferredUnloads - 1] = module;
-}
 
 /* Add a pointer to a new DriverRec to xf86DriverList */
 
 void
-xf86AddDriver(DriverPtr driver, pointer module, int flags)
+xf86AddDriver(DriverPtr driver, ModuleDescPtr module, int flags)
 {
     /* Don't add null entries */
     if (!driver)
@@ -177,17 +155,14 @@ xf86AddDriver(DriverPtr driver, pointer module, int flags)
 #endif
 
 void
-xf86DeleteDriver(int drvIndex, Bool deferUnload)
+xf86DeleteDriver(int drvIndex)
 {
     if (xf86DriverList[drvIndex] &&
 	!xf86DriverHasEntities(xf86DriverList[drvIndex])) {
 	xf86ClearDriverEntities(xf86DriverList[drvIndex]);
 #ifdef XFree86LOADER
 	if (xf86DriverList[drvIndex]->module) {
-	    if (deferUnload)
-		DeferredUnloadModule(xf86DriverList[drvIndex]->module);
-	    else
-		UnloadModule(xf86DriverList[drvIndex]->module);
+	    UnloadModule(xf86DriverList[drvIndex]->module);
 	    xfree(xf86DriverList[drvIndex]);
 	}
 #endif
@@ -199,7 +174,7 @@ xf86DeleteDriver(int drvIndex, Bool deferUnload)
 /* Add a pointer to a new InputDriverRec to xf86InputDriverList */
 
 void
-xf86AddInputDriver(InputDriverPtr driver, pointer module, int flags)
+xf86AddInputDriver(InputDriverPtr driver, ModuleDescPtr module, int flags)
 {
     /* Don't add null entries */
     if (!driver)
@@ -228,7 +203,7 @@ xf86DeleteInputDriver(int drvIndex)
 }
 
 void
-xf86AddModuleInfo(ModuleInfoPtr info, pointer module)
+xf86AddModuleInfo(ModuleInfoPtr info, ModuleDescPtr module)
 {
     /* Don't add null entries */
     if (!module)
@@ -343,7 +318,7 @@ xf86DeleteScreen(int scrnIndex, int flags)
 
 #ifdef XFree86LOADER
     if (pScrn->module)
-	DeferredUnloadModule(pScrn->module);
+	UnloadModule(pScrn->module);
 #endif
 
     if (pScrn->drv)
@@ -357,7 +332,7 @@ xf86DeleteScreen(int scrnIndex, int flags)
     xfree(pScrn);
 
     /* Move the other entries down, updating their scrnIndex fields */
-    
+
     xf86NumScreens--;
 
     for (i = scrnIndex; i < xf86NumScreens; i++) {
@@ -602,7 +577,7 @@ xf86SetDepthBpp(ScrnInfoPtr scrp, int depth, int dummy, int fbbpp,
 
     /* If none of these is set, pick a default */
     if (scrp->bitsPerPixel < 0 && scrp->depth < 0) {
-        if (fbbpp > 0 || depth > 0) {
+	if (fbbpp > 0 || depth > 0) {
 	    if (fbbpp > 0)
 		scrp->bitsPerPixel = fbbpp;
 	    if (depth > 0)
@@ -892,9 +867,9 @@ xf86SetWeight(ScrnInfoPtr scrp, rgb weight, rgb mask)
 	scrp->mask.red = mask.red;
 	scrp->mask.green = mask.green;
 	scrp->mask.blue = mask.blue;
-	scrp->offset.red = ffs(mask.red);
-	scrp->offset.green = ffs(mask.green);
-	scrp->offset.blue = ffs(mask.blue);
+	scrp->offset.red = ffs(mask.red) - 1;
+	scrp->offset.green = ffs(mask.green) - 1;
+	scrp->offset.blue = ffs(mask.blue) - 1;
     }
     return TRUE;
 }
@@ -936,12 +911,12 @@ xf86SetDefaultVisual(ScrnInfoPtr scrp, int visual)
 	if (scrp->depth == 1 && scrp->defaultVisual != StaticGray)
 	   bad = TRUE;
 #if 0
-        else if (scrp->depth == 4 &&
-                 (scrp->defaultVisual == TrueColor ||
-                  scrp->defaultVisual == DirectColor))
-           bad = TRUE;
+	else if (scrp->depth == 4 &&
+		 (scrp->defaultVisual == TrueColor ||
+		  scrp->defaultVisual == DirectColor))
+	   bad = TRUE;
 #endif
-        else if (scrp->depth > MAX_PSEUDO_DEPTH &&
+	else if (scrp->depth > MAX_PSEUDO_DEPTH &&
 		 scrp->defaultVisual != TrueColor &&
 		 scrp->defaultVisual != DirectColor)
 	   bad = TRUE;
@@ -971,7 +946,7 @@ xf86SetGamma(ScrnInfoPtr scrp, Gamma gamma)
 {
     MessageType from = X_DEFAULT;
 #if 0
-    xf86MonPtr DDC = (xf86MonPtr)(scrp->monitor->DDC); 
+    xf86MonPtr DDC = (xf86MonPtr)(scrp->monitor->DDC);
 #endif
     if (TEST_GAMMA(xf86Gamma)) {
 	from = X_CMDLINE;
@@ -985,7 +960,7 @@ xf86SetGamma(ScrnInfoPtr scrp, Gamma gamma)
 	scrp->gamma.blue = SET_GAMMA(scrp->monitor->gamma.blue);
 #if 0
     } else if ( DDC && DDC->features.gamma > GAMMA_ZERO ) {
-        from = X_PROBED;
+	from = X_PROBED;
 	scrp->gamma.red = SET_GAMMA(DDC->features.gamma);
 	scrp->gamma.green = SET_GAMMA(DDC->features.gamma);
 	scrp->gamma.blue = SET_GAMMA(DDC->features.gamma);
@@ -1024,7 +999,7 @@ void
 xf86SetDpi(ScrnInfoPtr pScrn, int x, int y)
 {
     MessageType from = X_DEFAULT;
-    xf86MonPtr DDC = (xf86MonPtr)(pScrn->monitor->DDC); 
+    xf86MonPtr DDC = (xf86MonPtr)(pScrn->monitor->DDC);
     int ddcWidthmm, ddcHeightmm;
     int widthErr, heightErr;
 
@@ -1034,7 +1009,7 @@ xf86SetDpi(ScrnInfoPtr pScrn, int x, int y)
 
     if (DDC && (DDC->features.hsize > 0 && DDC->features.vsize > 0) ) {
       /* DDC gives display size in mm for individual modes,
-       * but cm for monitor 
+       * but cm for monitor
        */
       ddcWidthmm = DDC->features.hsize * 10; /* 10mm in 1cm */
       ddcHeightmm = DDC->features.vsize * 10; /* 10mm in 1cm */
@@ -1078,7 +1053,7 @@ xf86SetDpi(ScrnInfoPtr pScrn, int x, int y)
 	  if (widthErr>10 || heightErr>10) {
 	    /* Should include config file name for monitor here */
 	    xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
-		       "Probed monitor is %dx%d mm, using Displaysize %dx%d mm\n", 
+		       "Probed monitor is %dx%d mm, using Displaysize %dx%d mm\n",
 		       ddcWidthmm,ddcHeightmm, pScrn->widthmm,pScrn->heightmm);
 	  }
 	}
@@ -1174,7 +1149,7 @@ xf86SetRootClip (ScreenPtr pScreen, Bool enable)
 	    pWin->valdata->before.resized = TRUE;
 	}
     }
-    
+
     /*
      * Use REGION_BREAK to avoid optimizations in ValidateTree
      * that assume the root borderClip can't change well, normally
@@ -1192,16 +1167,16 @@ xf86SetRootClip (ScreenPtr pScreen, Bool enable)
 	    REGION_RESET(pScreen, &pWin->borderClip, &box);
 	pWin->drawable.width = pScreen->width;
 	pWin->drawable.height = pScreen->height;
-        REGION_BREAK (pWin->drawable.pScreen, &pWin->clipList);
+	REGION_BREAK (pWin->drawable.pScreen, &pWin->clipList);
     }
     else
     {
 	REGION_EMPTY(pScreen, &pWin->borderClip);
 	REGION_BREAK (pWin->drawable.pScreen, &pWin->clipList);
     }
-    
+
     ResizeChildrenWinSize (pWin, 0, 0, 0, 0);
-    
+
     if (WasViewable)
     {
 	if (pWin->backStorage)
@@ -1246,7 +1221,7 @@ xf86SetRootClip (ScreenPtr pScreen, Bool enable)
 	if (bsExposed)
 	{
 	    RegionPtr	valExposed = NullRegion;
-    
+
 	    if (pWin->valdata)
 		valExposed = &pWin->valdata->after.exposed;
 	    (*pScreen->WindowExposures) (pWin, valExposed, bsExposed);
@@ -1305,7 +1280,7 @@ xf86EnableDisableFBAccess(int scrnIndex, Bool enable)
 	 */
 	pspix->devPrivate = pScrnInfo->pixmapPrivate;
 	/*
-	 * Restore all of the clip lists on the screen 
+	 * Restore all of the clip lists on the screen
 	 */
 	if (!xf86Resetting)
 	    xf86SetRootClip (pScreen, TRUE);
@@ -1314,7 +1289,7 @@ xf86EnableDisableFBAccess(int scrnIndex, Bool enable)
     else
     {
 	/*
-	 * Empty all of the clip lists on the screen 
+	 * Empty all of the clip lists on the screen
 	 */
 	xf86SetRootClip (pScreen, FALSE);
 	/*
@@ -1348,7 +1323,6 @@ xf86VDrvMsgVerb(int scrnIndex, MessageType type, int verb, const char *format,
     } else
 	LogVMessageVerb(type, verb, format, args);
 }
-#undef PREFIX_SIZE
 
 /* Print driver messages, with verbose level specified directly */
 void
@@ -1430,21 +1404,19 @@ xf86LogInit()
 
 #define LOGSUFFIX ".log"
 #define LOGOLDSUFFIX ".old"
-    
+
     /* Get the log file name */
     if (xf86FileCmdline.logFile) {
 	lf = xf86FileCmdline.logFile;
 	xf86FileCmdline.logFile = (char *)LogInit(lf, LOGOLDSUFFIX);
     } else {
 	/* Append the display number and ".log" */
-	lf = malloc(strlen(xf86FileDefaults.logFile) + strlen("%s") +
-		    strlen(LOGSUFFIX) + 1);
+	xasprintf(&lf, "%s%%s" LOGSUFFIX, xf86FileDefaults.logFile);
 	if (!lf)
-	    FatalError("Cannot allocate space for the log file name\n");
-	sprintf(lf, "%s%%s" LOGSUFFIX, xf86FileDefaults.logFile);
+	    FatalError("Cannot allocate space for the log file name.\n");
 	xf86FileDefaults.logFile = (char *)LogInit(lf, LOGOLDSUFFIX);
     }
-    free(lf);
+    xfree(lf);
     xf86LogFileWasOpened = TRUE;
 
     xf86SetVerbosity(xf86Verbose);
@@ -1560,7 +1532,7 @@ xf86MatchDevice(const char *drivername, GDevPtr **sectlist)
 	*sectlist = NULL;
 
     if (xf86DoProbe) return 1;
-  
+
     if (xf86DoConfigure && xf86DoConfigurePass1) return 1;
 
     /*
@@ -1568,18 +1540,18 @@ xf86MatchDevice(const char *drivername, GDevPtr **sectlist)
      * as they show up in the config file with the drivers that the server
      * loads at run time.
      *
-     * ChipProbe can call 
-     * int xf86MatchDevice(char * drivername, GDevPtr ** sectlist) 
+     * ChipProbe can call
+     * int xf86MatchDevice(char * drivername, GDevPtr ** sectlist)
      * with its driver name. The function allocates an array of GDevPtr and
      * returns this via sectlist and returns the number of elements in
      * this list as return value. 0 means none found, -1 means fatal error.
-     * 
+     *
      * It can figure out which of the Device sections to use for which card
      * (using things like the Card statement, etc). For single headed servers
      * there will of course be just one such Device section.
      */
     i = 0;
-    
+
     /*
      * first we need to loop over all the Screens sections to get to all
      * 'active' device sections
@@ -1587,17 +1559,17 @@ xf86MatchDevice(const char *drivername, GDevPtr **sectlist)
     for (j = 0; j < xf86Info.serverLayout->numScreens; j++) {
 	if (!xf86Info.serverLayout->screenLayouts[j])
 	    continue;
-        screensecptr = xf86Info.serverLayout->screenLayouts[j]->screen;
-        if (screensecptr && screensecptr->device &&
+	screensecptr = xf86Info.serverLayout->screenLayouts[j]->screen;
+	if (screensecptr && screensecptr->device &&
 	    screensecptr->device->driver &&
-            xf86NameCmp(screensecptr->device->driver,drivername) == 0 &&
-            !screensecptr->device->claimed) {
-            /*
-             * we have a matching driver that wasn't claimed, yet
-             */
-            pgdp = xnfrealloc(pgdp, (i + 2) * sizeof(GDevPtr));
-            pgdp[i++] = screensecptr->device;
-        }
+	    xf86NameCmp(screensecptr->device->driver,drivername) == 0 &&
+	    !screensecptr->device->claimed) {
+	    /*
+	     * we have a matching driver that wasn't claimed, yet
+	     */
+	    pgdp = xnfrealloc(pgdp, (i + 2) * sizeof(GDevPtr));
+	    pgdp[i++] = screensecptr->device;
+	}
     }
 
     /* Then handle the inactive devices */
@@ -1614,12 +1586,12 @@ xf86MatchDevice(const char *drivername, GDevPtr **sectlist)
 	    j++;
 	}
     }
-    
+
     /*
      * make the array NULL terminated and return its address
      */
     if (i)
-        pgdp[i] = NULL;
+	pgdp[i] = NULL;
 
     if (sectlist)
 	*sectlist = pgdp;
@@ -1638,7 +1610,7 @@ struct Inst {
 };
 
 int
-xf86MatchPciInstances(const char *driverName, int vendorID, 
+xf86MatchPciInstances(const char *driverName, int vendorID,
 		      SymTabPtr chipsets, PciChipsets *PCIchipsets,
 		      GDevPtr *devList, int numDevs, DriverPtr drvp,
 		      int **foundEntities)
@@ -1651,8 +1623,8 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
 	GDevPtr		dev;
 	Bool		foundHW;  /* PCIid in list of supported chipsets */
 	Bool		claimed;  /* BusID matches with a device section */
-        int             chip;
-        int		screen;
+	int             chip;
+	int		screen;
     } *instances = NULL;
     int numClaimedInstances = 0;
     int allocatedInstances = 0;
@@ -1664,39 +1636,43 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
     int *retEntities = NULL;
 
     *foundEntities = NULL;
+    if (xf86PciVideoInfo == NULL)
+	return 0;
 
-    if (vendorID == 0) {
-	for (ppPci = xf86PciVideoInfo; *ppPci != NULL; ppPci++) {
+    for (ppPci = xf86PciVideoInfo; (pPci = *ppPci); ppPci++) {
+	/* Two or more different drivers may not share entities */
+	if (!xf86CheckPciSlot(pPci->bus, pPci->device, pPci->func))
+	    continue;
+
+	if (vendorID == 0) {
 	    Bool foundVendor = FALSE;
 	    for (id = PCIchipsets; id->PCIid != -1; id++) {
-	        if ( (((id->PCIid & 0xFFFF0000) >> 16) == (*ppPci)->vendor)) {
+		if ( (((id->PCIid & 0xFFFF0000) >> 16) == pPci->vendor)) {
 		    if (!foundVendor) {
-	                ++allocatedInstances;
+			++allocatedInstances;
 			instances = xnfrealloc(instances,
 				     allocatedInstances * sizeof(struct Inst));
-			instances[allocatedInstances - 1].pci = *ppPci;
+			instances[allocatedInstances - 1].pci = pPci;
 			instances[allocatedInstances - 1].dev = NULL;
 			instances[allocatedInstances - 1].claimed = FALSE;
 			instances[allocatedInstances - 1].foundHW = FALSE;
 			instances[allocatedInstances - 1].screen = 0;
 			foundVendor = TRUE;
-		    } 
-		    if ((id->PCIid & 0x0000FFFF) == (*ppPci)->chipType) {
-	               instances[allocatedInstances - 1].foundHW = TRUE;
+		    }
+		    if ((id->PCIid & 0x0000FFFF) == pPci->chipType) {
+		       instances[allocatedInstances - 1].foundHW = TRUE;
 		       instances[allocatedInstances - 1].chip = id->numChipset;
 		       numFound++;
 		    }
 		}
 	    }
-	}
-    } else if (vendorID == PCI_VENDOR_GENERIC) {
-	for (ppPci = xf86PciVideoInfo; *ppPci != NULL; ppPci++) {
+	} else if (vendorID == PCI_VENDOR_GENERIC) {
 	    for (id = PCIchipsets; id->PCIid != -1; id++) {
-		if (id->PCIid == xf86CheckPciGAType(*ppPci)) {
+		if (id->PCIid == xf86CheckPciGAType(pPci)) {
 		    ++allocatedInstances;
 		    instances = xnfrealloc(instances,
 				  allocatedInstances * sizeof(struct Inst));
-		    instances[allocatedInstances - 1].pci = *ppPci;
+		    instances[allocatedInstances - 1].pci = pPci;
 		    instances[allocatedInstances - 1].dev = NULL;
 		    instances[allocatedInstances - 1].claimed = FALSE;
 		    instances[allocatedInstances - 1].foundHW = TRUE;
@@ -1705,24 +1681,21 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
 		    numFound++;
 		}
 	    }
-	}
-    } else {
-	/* Find PCI devices that match the given vendor ID */
-	for (ppPci = xf86PciVideoInfo; (ppPci != NULL)
-	       && (*ppPci != NULL); ppPci++) {
-	    if ((*ppPci)->vendor == vendorID) {
+	} else {
+	    /* Find PCI devices that match the given vendor ID */
+	    if (pPci->vendor == vendorID) {
 		++allocatedInstances;
 		instances = xnfrealloc(instances,
 			      allocatedInstances * sizeof(struct Inst));
-		instances[allocatedInstances - 1].pci = *ppPci;
+		instances[allocatedInstances - 1].pci = pPci;
 		instances[allocatedInstances - 1].dev = NULL;
 		instances[allocatedInstances - 1].claimed = FALSE;
 		instances[allocatedInstances - 1].foundHW = FALSE;
-	        instances[allocatedInstances - 1].screen = 0;
+		instances[allocatedInstances - 1].screen = 0;
 
 		/* Check if the chip type is listed in the chipsets table */
 		for (id = PCIchipsets; id->PCIid != -1; id++) {
-		    if (id->PCIid == (*ppPci)->chipType) {
+		    if (id->PCIid == pPci->chipType) {
 			instances[allocatedInstances - 1].chip
 			    = id->numChipset;
 			instances[allocatedInstances - 1].foundHW = TRUE;
@@ -1758,7 +1731,7 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
 		if (!xf86CheckPciSlot(pPci->bus, pPci->device, pPci->func))
 		    continue;
 		actualcards++;
-	    	pGDev = xf86AddDeviceToConfigure(drvp->driverName,
+		pGDev = xf86AddDeviceToConfigure(drvp->driverName,
 						 instances[i].pci, -1);
 		if (pGDev) {
 		   /*
@@ -1786,20 +1759,20 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
     */
 
     for (j = 0; j < numDevs; j++) {
-        if (devList[j]->screen > 0 && devList[j]->busID 
+	if (devList[j]->screen > 0 && devList[j]->busID
 	    && *devList[j]->busID) {
 	    for (i = 0; i < allocatedInstances; i++) {
-	        pPci = instances[i].pci;
-	        if (xf86ComparePciBusString(devList[j]->busID, pPci->bus,
+		pPci = instances[i].pci;
+		if (xf86ComparePciBusString(devList[j]->busID, pPci->bus,
 					    pPci->device,
 					    pPci->func)) {
 		    allocatedInstances++;
 		    instances = xnfrealloc(instances,
-					   allocatedInstances * 
+					   allocatedInstances *
 					   sizeof(struct Inst));
 		    instances[allocatedInstances - 1] = instances[i];
 		    instances[allocatedInstances - 1].screen =
-		      				devList[j]->screen;
+						devList[j]->screen;
 		    numFound++;
 		    break;
 		}
@@ -1817,19 +1790,19 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
 					   pPci->device,
 					   pPci->func) &&
 		    devList[j]->screen == instances[i].screen) {
-		   
+
 		    if (devBus)
-                        xf86MsgVerb(X_WARNING,0,
+			xf86MsgVerb(X_WARNING,0,
 			    "%s: More than one matching Device section for "
 			    "instances\n\t(BusID: %s) found: %s\n",
 			    driverName, devList[j]->busID,
 			    devList[j]->identifier);
 		    else
 			devBus = devList[j];
-		} 
+		}
 	    } else {
-		/* 
-		 * if device section without BusID is found 
+		/*
+		 * if device section without BusID is found
 		 * only assign to it to the primary device.
 		 */
 		if (xf86IsPrimaryPci(pPci)) {
@@ -1844,7 +1817,7 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
 		}
 	    }
 	}
-	if (devBus) dev = devBus;  /* busID preferred */ 
+	if (devBus) dev = devBus;  /* busID preferred */
 	if (!dev) {
 	    if (xf86CheckPciSlot(pPci->bus, pPci->device, pPci->func)) {
 		xf86MsgVerb(X_WARNING, 0, "%s: No matching Device section "
@@ -1945,13 +1918,13 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
      */
     numFound = 0;
     for (i = 0; i < allocatedInstances && numClaimedInstances > 0; i++) {
-	
+
 	if (!instances[i].claimed)
 	    continue;
 	pPci = instances[i].pci;
 
 
-        /*
+	/*
 	 * Allow the same entity to be used more than once for devices with
 	 * multiple screens per entity.  This assumes implicitly that there
 	 * will be a screen == 0 instance.
@@ -1959,7 +1932,7 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
 	 * XXX Need to make sure that two different drivers don't claim
 	 * the same screen > 0 instance.
 	 */
-        if (instances[i].screen == 0 &&
+	if (instances[i].screen == 0 &&
 	    !xf86CheckPciSlot(pPci->bus, pPci->device, pPci->func))
 	    continue;
 
@@ -1967,7 +1940,7 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
 	ErrorF("%s: card at %d:%d:%d is claimed by a Device section\n",
 	       driverName, pPci->bus, pPci->device, pPci->func);
 #endif
-	
+
 	/* Allocate an entry in the lists to be returned */
 	numFound++;
 	retEntities = xnfrealloc(retEntities, numFound * sizeof(int));
@@ -1976,12 +1949,12 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
 			       pPci->func,drvp,	instances[i].chip,
 			       instances[i].dev,instances[i].dev->active ?
 			       TRUE : FALSE);
-        if (retEntities[numFound - 1] == -1 && instances[i].screen > 0) {
+	if (retEntities[numFound - 1] == -1 && instances[i].screen > 0) {
 	    for (j = 0; j < xf86NumEntities; j++) {
-	        EntityPtr pEnt = xf86Entities[j];
-	        if (pEnt->busType != BUS_PCI)
+		EntityPtr pEnt = xf86Entities[j];
+		if (pEnt->busType != BUS_PCI)
 		    continue;
-	        if (pEnt->pciBusId.bus == pPci->bus &&
+		if (pEnt->pciBusId.bus == pPci->bus &&
 		    pEnt->pciBusId.device == pPci->device &&
 		    pEnt->pciBusId.func == pPci->func) {
 		    retEntities[numFound - 1] = j;
@@ -1995,7 +1968,7 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
     if (numFound > 0) {
 	*foundEntities = retEntities;
     }
-	
+
     return numFound;
 }
 
@@ -2063,18 +2036,18 @@ xf86MatchIsaInstances(const char *driverName, SymTabPtr chipsets,
 				dev->identifier);
 		} else
 		    foundChip = c->token;
-	    } else { 
+	    } else {
 		if (FindIsaDevice) foundChip = (*FindIsaDevice)(dev);
-                                                        /* Probe it */
+							/* Probe it */
 		from = X_PROBED;
 	    }
 	}
-	
+
 	/* Check if the chip type is listed in the chipset table - for sanity*/
 
 	if (foundChip >= 0){
 	    for (Chips = ISAchipsets; Chips->numChipset >= 0; Chips++) {
-		if (Chips->numChipset == foundChip) 
+		if (Chips->numChipset == foundChip)
 		    break;
 	    }
 	    if (Chips->numChipset == -1){
@@ -2097,7 +2070,7 @@ xf86MatchIsaInstances(const char *driverName, SymTabPtr chipsets,
 	}
     }
     *foundEntities = retEntities;
-    
+
     return numFound;
 }
 
@@ -2121,7 +2094,7 @@ xf86GetClocks(ScrnInfoPtr pScrn, int num, Bool (*ClockFunc)(ScrnInfoPtr, int),
     if (num > MAXCLOCKS)
 	num = MAXCLOCKS;
 
-    for (i = 0; i < num; i++) 
+    for (i = 0; i < num; i++)
     {
 	if (ProtectRegs)
 	    (*ProtectRegs)(pScrn, TRUE);
@@ -2134,50 +2107,50 @@ xf86GetClocks(ScrnInfoPtr pScrn, int num, Bool (*ClockFunc)(ScrnInfoPtr, int),
 	    (*ProtectRegs)(pScrn, FALSE);
 	if (BlankScreen)
 	    (*BlankScreen)(pScrn, FALSE);
-	    
-    	usleep(50000);     /* let VCO stabilise */
 
-    	cnt  = 0;
-    	sync = 200000;
+	usleep(50000);     /* let VCO stabilise */
+
+	cnt  = 0;
+	sync = 200000;
 
 	/* XXX How critical is this? */
-    	if (!xf86DisableInterrupts())
-    	{
+	if (!xf86DisableInterrupts())
+	{
 	    (*ClockFunc)(pScrn, CLK_REG_RESTORE);
 	    ErrorF("Failed to disable interrupts during clock probe.  If\n");
 	    ErrorF("your OS does not support disabling interrupts, then you\n");
 	    FatalError("must specify a Clocks line in the XF86Config file.\n");
 	}
-	while ((inb(status) & maskval) == 0x00) 
+	while ((inb(status) & maskval) == 0x00)
 	    if (sync-- == 0) goto finish;
 	/* Something appears to be happening, so reset sync count */
 	sync = 200000;
-	while ((inb(status) & maskval) == maskval) 
+	while ((inb(status) & maskval) == maskval)
 	    if (sync-- == 0) goto finish;
 	/* Something appears to be happening, so reset sync count */
 	sync = 200000;
-	while ((inb(status) & maskval) == 0x00) 
+	while ((inb(status) & maskval) == 0x00)
 	    if (sync-- == 0) goto finish;
-    
-	for (rcnt = 0; rcnt < 5; rcnt++) 
+
+	for (rcnt = 0; rcnt < 5; rcnt++)
 	{
-	    while (!(inb(status) & maskval)) 
+	    while (!(inb(status) & maskval))
 		cnt++;
-	    while ((inb(status) & maskval)) 
+	    while ((inb(status) & maskval))
 		cnt++;
 	}
-    
+
 finish:
 	xf86EnableInterrupts();
 
 	pScrn->clock[i] = cnt ? cnt : -1;
 	if (BlankScreen)
-            (*BlankScreen)(pScrn, TRUE);
+	    (*BlankScreen)(pScrn, TRUE);
     }
 
     xf86SetPriority(FALSE);
 
-    for (i = 0; i < num; i++) 
+    for (i = 0; i < num; i++)
     {
 	if (i != knownclkindex)
 	{
@@ -2185,11 +2158,11 @@ finish:
 	    {
 		pScrn->clock[i] = 0;
 	    }
-	    else 
+	    else
 	    {
 		pScrn->clock[i] = (int)(0.5 +
-                    (((float)knownclkvalue) * pScrn->clock[knownclkindex]) / 
-	            (pScrn->clock[i]));
+		    (((float)knownclkvalue) * pScrn->clock[knownclkindex]) /
+		    (pScrn->clock[i]));
 		/* Round to nearest 10KHz */
 		pScrn->clock[i] += 5;
 		pScrn->clock[i] /= 10;
@@ -2199,7 +2172,7 @@ finish:
     }
 
     pScrn->clock[knownclkindex] = knownclkvalue;
-    pScrn->numClocks = num; 
+    pScrn->numClocks = num;
 
     /* Restore registers that were written on */
     (*ClockFunc)(pScrn, CLK_REG_RESTORE);
@@ -2387,7 +2360,7 @@ xf86GetVersion()
 }
 
 CARD32
-xf86GetModuleVersion(pointer module)
+xf86GetModuleVersion(ModuleDescPtr module)
 {
 #ifdef XFree86LOADER
     return (CARD32)LoaderGetModuleVersion(module);
@@ -2396,11 +2369,29 @@ xf86GetModuleVersion(pointer module)
 #endif
 }
 
-pointer
+ModuleDescPtr
+xf86GetSubModuleByName(ModuleDescPtr mod, const char *name)
+{
+#ifdef XFree86LOADER
+    return LoaderGetSubModuleByName(mod, name);
+#else
+    return NULL;
+#endif
+}
+
+void
+xf86SetParentModuleRequirements(ModuleDescPtr module, XF86ModReqInfo *req)
+{
+#ifdef XFree86LOADER
+    LoaderSetParentModuleRequirements(module, req);
+#endif
+}
+
+ModuleDescPtr
 xf86LoadDrvSubModule(DriverPtr drv, const char *name)
 {
 #ifdef XFree86LOADER
-    pointer ret;
+    ModuleDescPtr ret;
     int errmaj = 0, errmin = 0;
 
     ret = LoadSubModule(drv->module, name, NULL, NULL, NULL, NULL,
@@ -2413,11 +2404,29 @@ xf86LoadDrvSubModule(DriverPtr drv, const char *name)
 #endif
 }
 
-pointer
+ModuleDescPtr
+xf86LoadDrvSubModuleWithRequirements(DriverPtr drv, const char *name,
+				     XF86ModReqInfo *req)
+{
+#ifdef XFree86LOADER
+    ModuleDescPtr ret;
+    int errmaj = 0, errmin = 0;
+
+    ret = LoadSubModule(drv->module, name, NULL, NULL, NULL, req,
+			&errmaj, &errmin);
+    if (!ret)
+	LoaderErrorMsg(NULL, name, errmaj, errmin);
+    return ret;
+#else
+    return (pointer)1;
+#endif
+}
+
+ModuleDescPtr
 xf86LoadSubModule(ScrnInfoPtr pScrn, const char *name)
 {
 #ifdef XFree86LOADER
-    pointer ret;
+    ModuleDescPtr ret;
     int errmaj = 0, errmin = 0;
 
     ret = LoadSubModule(pScrn->module, name, NULL, NULL, NULL, NULL,
@@ -2430,21 +2439,39 @@ xf86LoadSubModule(ScrnInfoPtr pScrn, const char *name)
 #endif
 }
 
+ModuleDescPtr
+xf86LoadSubModuleWithRequirements(ScrnInfoPtr pScrn, const char *name,
+				  XF86ModReqInfo *req)
+{
+#ifdef XFree86LOADER
+    ModuleDescPtr ret;
+    int errmaj = 0, errmin = 0;
+
+    ret = LoadSubModule(pScrn->module, name, NULL, NULL, NULL, req,
+			&errmaj, &errmin);
+    if (!ret)
+	LoaderErrorMsg(pScrn->name, name, errmaj, errmin);
+    return ret;
+#else
+    return (pointer)1;
+#endif
+}
+
 /*
  * xf86LoadOneModule loads a single module.
- */             
-pointer
+ */
+ModuleDescPtr
 xf86LoadOneModule(char *name, pointer opt)
 {
 #ifdef XFree86LOADER
     int errmaj, errmin;
 #endif
     char *Name;
-    pointer mod;
-    
+    ModuleDescPtr mod;
+
     if (!name)
 	return NULL;
-    
+
 #ifndef NORMALISE_MODULE_NAME
     Name = xstrdup(name);
 #else
@@ -2472,13 +2499,9 @@ xf86LoadOneModule(char *name, pointer opt)
 }
 
 void
-xf86UnloadSubModule(pointer mod)
+xf86UnloadSubModule(ModuleDescPtr mod)
 {
-    /*
-     * This is disabled for now.  The loader isn't smart enough yet to undo
-     * relocations.
-     */
-#if defined(XFree86LOADER) && 0
+#if defined(XFree86LOADER)
     UnloadSubModule(mod);
 #endif
 }
@@ -2500,7 +2523,7 @@ xf86LoaderReqSymLists(const char **list0, ...)
     va_list ap;
 
     va_start(ap, list0);
-    LoaderVReqSymLists(list0, ap);
+    LoaderVReqSymLists(NULL, list0, ap);
     va_end(ap);
 #endif
 }
@@ -2512,7 +2535,7 @@ xf86LoaderReqSymbols(const char *sym0, ...)
     va_list ap;
 
     va_start(ap, sym0);
-    LoaderVReqSymbols(sym0, ap);
+    LoaderVReqSymbols(NULL, sym0, ap);
     va_end(ap);
 #endif
 }
@@ -2524,7 +2547,7 @@ xf86LoaderRefSymLists(const char **list0, ...)
     va_list ap;
 
     va_start(ap, list0);
-    LoaderVRefSymLists(list0, ap);
+    LoaderVRefSymLists(NULL, list0, ap);
     va_end(ap);
 #endif
 }
@@ -2536,7 +2559,59 @@ xf86LoaderRefSymbols(const char *sym0, ...)
     va_list ap;
 
     va_start(ap, sym0);
-    LoaderVRefSymbols(sym0, ap);
+    LoaderVRefSymbols(NULL, sym0, ap);
+    va_end(ap);
+#endif
+}
+
+int
+xf86LoaderModReqSymLists(ModuleDescPtr module, const char **list0, ...)
+{
+    int ret = 0;
+#ifdef XFree86LOADER
+    va_list ap;
+
+    va_start(ap, list0);
+    ret = LoaderVReqSymLists(module, list0, ap);
+    va_end(ap);
+#endif
+    return ret;
+}
+
+int
+xf86LoaderModReqSymbols(ModuleDescPtr module, const char *sym0, ...)
+{
+    int ret = 0;
+#ifdef XFree86LOADER
+    va_list ap;
+
+    va_start(ap, sym0);
+    ret = LoaderVReqSymbols(module, sym0, ap);
+    va_end(ap);
+#endif
+    return ret;
+}
+
+void
+xf86LoaderModRefSymLists(ModuleDescPtr module, const char **list0, ...)
+{
+#ifdef XFree86LOADER
+    va_list ap;
+
+    va_start(ap, list0);
+    LoaderVRefSymLists(module, list0, ap);
+    va_end(ap);
+#endif
+}
+
+void
+xf86LoaderModRefSymbols(ModuleDescPtr module, const char *sym0, ...)
+{
+#ifdef XFree86LOADER
+    va_list ap;
+
+    va_start(ap, sym0);
+    LoaderVRefSymbols(module, sym0, ap);
     va_end(ap);
 #endif
 }
@@ -2551,7 +2626,7 @@ static const OptionInfoRec BSOptions[] = {
    { -1,                   NULL,           OPTV_NONE,    {0}, FALSE }
 };
 
-void 
+void
 xf86SetBackingStore(ScreenPtr pScreen)
 {
     Bool useBS = FALSE;
@@ -2591,7 +2666,7 @@ static const OptionInfoRec SMOptions[] = {
    { -1,                   NULL,           OPTV_NONE,    {0}, FALSE }
 };
 
-void 
+void
 xf86SetSilkenMouse (ScreenPtr pScreen)
 {
     Bool useSM = TRUE;
@@ -2602,17 +2677,17 @@ xf86SetSilkenMouse (ScreenPtr pScreen)
     options = xnfalloc(sizeof(SMOptions));
     (void)memcpy(options, SMOptions, sizeof(SMOptions));
     xf86ProcessOptions(pScrn->scrnIndex, pScrn->options, options);
-    
+
     /* check for commandline option here */
     /* disable if screen shares resources */
-    if (((pScrn->racMemFlags & RAC_CURSOR) && 
+    if (((pScrn->racMemFlags & RAC_CURSOR) &&
 	 !xf86NoSharedResources(pScrn->scrnIndex,MEM)) ||
-	((pScrn->racIoFlags & RAC_CURSOR) && 
+	((pScrn->racIoFlags & RAC_CURSOR) &&
 	 !xf86NoSharedResources(pScrn->scrnIndex,IO))) {
 	useSM = FALSE;
 	from = X_PROBED;
     } else if (xf86silkenMouseDisableFlag) {
-        from = X_CMDLINE;
+	from = X_CMDLINE;
 	useSM = FALSE;
     } else {
 	if (xf86GetOptValBool(options, OPTION_SILKEN_MOUSE, &useSM))
@@ -2682,7 +2757,7 @@ xf86ConfigIsaEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
     IsaChipsets *i_id;
     EntityInfoPtr pEnt = xf86GetEntityInfo(entityIndex);
     if (!pEnt) return pScrn;
-    
+
     if (!(pEnt->location.type == BUS_ISA)) {
 	xfree(pEnt);
 	return pScrn;
@@ -2695,7 +2770,7 @@ xf86ConfigIsaEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
     }
 
     if (!pScrn)
-	pScrn = xf86AllocateScreen(pEnt->driver,scrnFlag); 
+	pScrn = xf86AllocateScreen(pEnt->driver,scrnFlag);
     xf86AddEntityToScreen(pScrn,entityIndex);
 
     if (i_chip) {
@@ -2720,7 +2795,7 @@ xf86ConfigPciEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
     EntityInfoPtr pEnt = xf86GetEntityInfo(entityIndex);
     if (!pEnt) return pScrn;
 
-    if (!(pEnt->location.type == BUS_PCI) 
+    if (!(pEnt->location.type == BUS_PCI)
 	|| !xf86GetPciInfoForEntity(entityIndex)) {
 	xfree(pEnt);
 	return pScrn;
@@ -2734,11 +2809,11 @@ xf86ConfigPciEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
     if (!pScrn)
 	pScrn = xf86AllocateScreen(pEnt->driver,scrnFlag);
     if (xf86IsEntitySharable(entityIndex)) {
-        xf86SetEntityShared(entityIndex);
+	xf86SetEntityShared(entityIndex);
     }
     xf86AddEntityToScreen(pScrn,entityIndex);
     if (xf86IsEntityShared(entityIndex)) {
-        return pScrn;
+	return pScrn;
     }
     if (p_chip) {
 	for (p_id = p_chip; p_id->numChipset != -1; p_id++) {
@@ -2756,12 +2831,12 @@ xf86ConfigPciEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
 
 ScrnInfoPtr
 xf86ConfigFbEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
-		   EntityProc init, EntityProc enter, EntityProc leave, 
+		   EntityProc init, EntityProc enter, EntityProc leave,
 		   pointer private)
 {
     EntityInfoPtr pEnt = xf86GetEntityInfo(entityIndex);
     if (!pEnt) return pScrn;
-    
+
     if (!(pEnt->location.type == BUS_NONE)) {
 	xfree(pEnt);
 	return pScrn;
@@ -2773,7 +2848,7 @@ xf86ConfigFbEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
     }
 
     if (!pScrn)
-	pScrn = xf86AllocateScreen(pEnt->driver,scrnFlag); 
+	pScrn = xf86AllocateScreen(pEnt->driver,scrnFlag);
     xf86AddEntityToScreen(pScrn,entityIndex);
 
     xf86SetEntityFuncs(entityIndex,init,enter,leave,private);
@@ -2789,61 +2864,61 @@ xf86ConfigFbEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
  */
 Bool
 xf86ConfigActiveIsaEntity(ScrnInfoPtr pScrn, int entityIndex,
-                          IsaChipsets *i_chip, resList res, EntityProc init,
-                          EntityProc enter, EntityProc leave, pointer private)
+			  IsaChipsets *i_chip, resList res, EntityProc init,
+			  EntityProc enter, EntityProc leave, pointer private)
 {
     IsaChipsets *i_id;
     EntityInfoPtr pEnt = xf86GetEntityInfo(entityIndex);
     if (!pEnt) return FALSE;
- 
+
     if (!pEnt->active || !(pEnt->location.type == BUS_ISA)) {
-        xfree(pEnt);
-        return FALSE;
+	xfree(pEnt);
+	return FALSE;
     }
- 
+
     xf86AddEntityToScreen(pScrn,entityIndex);
- 
+
     if (i_chip) {
-        for (i_id = i_chip; i_id->numChipset != -1; i_id++) {
-            if (pEnt->chipset == i_id->numChipset) break;
-        }
-        xf86ClaimFixedResources(i_id->resList,entityIndex);
+	for (i_id = i_chip; i_id->numChipset != -1; i_id++) {
+	    if (pEnt->chipset == i_id->numChipset) break;
+	}
+	xf86ClaimFixedResources(i_id->resList,entityIndex);
     }
     xfree(pEnt);
     xf86ClaimFixedResources(res,entityIndex);
     if (!xf86SetEntityFuncs(entityIndex,init,enter,leave,private))
-        return FALSE;
- 
+	return FALSE;
+
     return TRUE;
 }
- 
+
 Bool
 xf86ConfigActivePciEntity(ScrnInfoPtr pScrn, int entityIndex,
-                          PciChipsets *p_chip, resList res, EntityProc init,
-                          EntityProc enter, EntityProc leave, pointer private)
+			  PciChipsets *p_chip, resList res, EntityProc init,
+			  EntityProc enter, EntityProc leave, pointer private)
 {
     PciChipsets *p_id;
     EntityInfoPtr pEnt = xf86GetEntityInfo(entityIndex);
     if (!pEnt) return FALSE;
- 
+
     if (!pEnt->active || !(pEnt->location.type == BUS_PCI)) {
-        xfree(pEnt);
-        return FALSE;
+	xfree(pEnt);
+	return FALSE;
     }
     xf86AddEntityToScreen(pScrn,entityIndex);
- 
+
     if (p_chip) {
-        for (p_id = p_chip; p_id->numChipset != -1; p_id++) {
-            if (pEnt->chipset == p_id->numChipset) break;
-        }
-        xf86ClaimFixedResources(p_id->resList,entityIndex);
+	for (p_id = p_chip; p_id->numChipset != -1; p_id++) {
+	    if (pEnt->chipset == p_id->numChipset) break;
+	}
+	xf86ClaimFixedResources(p_id->resList,entityIndex);
     }
     xfree(pEnt);
- 
+
     xf86ClaimFixedResources(res,entityIndex);
     if (!xf86SetEntityFuncs(entityIndex,init,enter,leave,private))
-        return FALSE;
- 
+	return FALSE;
+
     return TRUE;
 }
 
@@ -2899,7 +2974,7 @@ xf86ConfigIsaEntityInactive(EntityInfoPtr pEnt, IsaChipsets *i_chip,
 }
 
 void
-xf86ConfigFbEntityInactive(EntityInfoPtr pEnt, EntityProc init, 
+xf86ConfigFbEntityInactive(EntityInfoPtr pEnt, EntityProc init,
 			   EntityProc enter, EntityProc leave, pointer private)
 {
     ScrnInfoPtr pScrn;
@@ -2932,7 +3007,8 @@ xf86RegisterRootWindowProperty(int ScrnIndex, Atom property, Atom type,
 
 #ifdef DEBUG
     ErrorF("xf86RegisterRootWindowProperty(%d, %ld, %ld, %d, %ld, %p)\n",
-	   ScrnIndex, property, type, format, len, value);
+	   ScrnIndex, (unsigned long)property, (unsigned long)type,
+	   format, len, value);
 #endif
 
     if (ScrnIndex<0 || ScrnIndex>=xf86NumScreens) {
@@ -2968,7 +3044,7 @@ xf86RegisterRootWindowProperty(int ScrnIndex, Atom property, Atom type,
     pNewProp->format = format;
     pNewProp->size = len;
     pNewProp->data = value;
- 
+
 #ifdef DEBUG
     ErrorF("new property filled\n");
 #endif
@@ -3003,14 +3079,14 @@ xf86RegisterRootWindowProperty(int ScrnIndex, Atom property, Atom type,
 	  ErrorF("- next %p\n", (void *)pRegProp);
 #endif
 	  pRegProp = pRegProp->next;
-        }
+	}
 	pRegProp->next = pNewProp;
       }
     }
 #ifdef DEBUG
     ErrorF("xf86RegisterRootWindowProperty succeeded\n");
 #endif
-    return(Success);    
+    return(Success);
 }
 
 Bool
